
"""
Usage::

    import install_cmake
    install_cmake.install()

"""

import os
import shutil
import sys
import zipfile

try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen

DEFAULT_CMAKE_VERSION = "3.5.2"


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[appveyor:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def _env_prepend(key, *values):
    os.environ[key] = os.pathsep.join(
        list(values) + os.environ.get(key, "").split(os.pathsep))


def install(cmake_version=DEFAULT_CMAKE_VERSION):
    """Download and install CMake into ``C:\\cmake``.

    The function also make sure to prepend ``C:\\cmake\\bin``
    to the ``PATH``."""

    cmake_version_major = cmake_version.split(".")[0]
    cmake_version_minor = cmake_version.split(".")[1]
    cmake_directory = "C:\\cmake-{}".format(cmake_version)
    cmake_package = "cmake-{}-win32-x86.zip".format(cmake_version)

    if not os.path.exists(cmake_directory):

        _log("Downloading", cmake_package)
        remote_file = urlopen(
            "https://cmake.org/files/v{}.{}/{}".format(
                cmake_version_major, cmake_version_minor, cmake_package))

        with open("C:\\%s" % cmake_package, "wb") as local_file:
            shutil.copyfileobj(remote_file, local_file)

        _log("Making directory", cmake_directory)
        try:
            os.mkdir(cmake_directory)
        except OSError:
            pass

        _log("Unpacking", cmake_package)
        with zipfile.ZipFile("C:\\%s" % cmake_package) as local_zip:
            local_zip.extractall(cmake_directory)

    else:
        _log("Skipping download: Directory %s exists" % cmake_package)

    _log("Updating PATH with", cmake_directory)
    _env_prepend("PATH", "%s\bin" % cmake_directory)


if __name__ == '__main__':
    install(sys.argv[1] if len(sys.argv) > 1 else DEFAULT_CMAKE_VERSION)




import os


def captured_lines(cap):
    """Given a ``capsys`` or ``capfd`` pytest fixture, return
     a tuple of the form ``(out_lines, error_lines)``.

    See http://doc.pytest.org/en/latest/capture.html
    """
    out, err = cap.readouterr()
    return (out.replace(os.linesep, "\n").split("\n"),
            err.replace(os.linesep, "\n").split("\n"))




"""
Usage::

    import tweak_environment
    tweak_environment.install()

"""

import os
import sys

from xml.dom.minidom import parse


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[appveyor:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def tweak():
    update_notepad_settings()


def update_notepad_settings():
    """Update notepad++ settings to facilitate its use.

    Settings updated:

     - ``TabSetting.replaceBySpace`` set to ``yes``

    """

    settings = "%s/Notepad++/config.xml" % os.environ["appdata"]

    if not os.path.exists(settings):
        _log("[notepad++] Skipping. Setting file not found:", settings)

    _log("notepad++: Parsing", settings)
    dom = parse(settings)

    _log("[notepad++] Updating")
    configs = dom.documentElement.getElementsByTagName('GUIConfigs').item(0)
    for elem in configs.getElementsByTagName('GUIConfig'):
        if elem.getAttribute('name') == "TabSetting":
            elem.setAttribute("replaceBySpace", "yes")

    _log("[notepad++] Writing", settings)
    with open(settings, "w") as output:
        dom.writexml(output)


if __name__ == '__main__':
    tweak()




"""
Usage::

    import install_pyenv
    install_pyenv.install("3.4.5")

"""

import os
import sys

from subprocess import check_call


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[travis:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def install(py_version):
    """Download and install ``pyenv``."""

    _log("Updating pyenv using brew")
    check_call(
        "\n".join((
            "brew update",
            "brew outdated pyenv || brew upgrade pyenv"
        )),
        shell=True
    )

    _log("Checking that requested python version", py_version, "works")
    check_call(
        "\n".join((
            "eval \"$( pyenv init - )\"",
            "pyenv install " + py_version,
            "pyenv local " + py_version,
            "echo 'Python Version:'",
            (
                "python -c \""
                "import sys, struct ; "
                "print(sys.version) ; "
                "print('{}-bit'.format(struct.calcsize('P') * 8))"
                "\""
            )
        )),
        shell=True
    )


if __name__ == '__main__':
    install(os.environ['PYTHONVERSION'])




"""
Usage::

    import install_cmake
    install_cmake.install()

"""

import os
import platform
import sys

from subprocess import check_call

DEFAULT_CMAKE_VERSION = "3.5.0"


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[travis:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def install(cmake_version=DEFAULT_CMAKE_VERSION, is_darwin=False):
    """Download and install CMake into ``/usr/local``."""

    cmake_os = "Darwin" if is_darwin else "Linux"
    cmake_name = "cmake-{}-{}-x86_64".format(cmake_version, cmake_os)
    cmake_package = ".".join((cmake_name, "tar", "gz"))
    cmake_version_major = cmake_version.split(".")[0]
    cmake_version_minor = cmake_version.split(".")[1]

    download_dir = os.environ["HOME"] + "/downloads"
    downloaded_package = os.path.join(download_dir, cmake_package)

    if not os.path.exists(downloaded_package):

        _log("Making directory: ", download_dir)
        try:
            os.mkdir(download_dir)
        except OSError:
            pass

        _log("Downloading", cmake_package)
        check_call([
            "wget", "--no-check-certificate", "--progress=dot",
            "https://cmake.org/files/v{}.{}/{}".format(
                cmake_version_major, cmake_version_minor, cmake_package),
            "-P", download_dir
        ])

    else:
        _log("Skipping download: Found ", downloaded_package)

    _log("Extracting", downloaded_package)
    check_call(["tar", "xzf", downloaded_package, '-C', download_dir])

    if is_darwin:
        prefix = "/usr/local/bin"
        _log("Removing any existing CMake in", prefix)
        check_call(
            ["sudo", "rm", "-f"] + [
                "/".join((prefix, subdir)) for subdir in
                ("cmake", "cpack", "cmake-gui", "ccmake", "ctest")
                ]
        )

        _log("Installing CMake in", prefix)
        check_call([
            "sudo",
            download_dir + "/" + cmake_name
            + "/CMake.app/Contents/bin/cmake-gui",
            "--install"
        ])

    else:
        home = os.environ["HOME"]
        assert os.path.exists(home)
        _log("Copying", download_dir + "/" + cmake_name, "to", home)
        check_call([
            "rsync", "-avz",
            download_dir + "/" + cmake_name + "/", home])


if __name__ == '__main__':
    install(sys.argv[1] if len(sys.argv) > 1 else DEFAULT_CMAKE_VERSION,
            is_darwin=platform.system().lower() == "darwin")




"""
Usage::

    import noop
    noop.execute()

"""

import os
import sys


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[generic:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def execute():
    """Display value of ``sys.argv`` and exit.
    """
    _log("Display value of 'sys.argv'")
    print(" ".join(sys.argv))

if __name__ == '__main__':
    execute()




import ci_addons
import os
import pytest
import subprocess

from . import captured_lines


def test_home():
    expected_home = os.path.abspath(os.path.dirname(__file__) + '/..')
    assert ci_addons.home() == expected_home


@pytest.mark.parametrize("addon", ['anyci/noop', 'anyci/noop.py'])
def test_path(addon):
    expected_path = os.path.join(ci_addons.home(), addon)
    if not addon.endswith('.py'):
        expected_path += '.py'
    assert ci_addons.path(addon) == expected_path


def test_addons():
    addons = ci_addons.addons()
    assert 'anyci' + os.path.sep + 'noop.py' in addons


@pytest.mark.parametrize("addon", ['anyci/noop', 'anyci/noop.py'])
def test_execute(addon, capfd):
    ci_addons.execute(addon, ['foo', 'bar'])
    output_lines, _ = captured_lines(capfd)
    noop_path = os.path.join(ci_addons.home(), 'anyci/noop.py')
    assert noop_path + ' foo bar' in output_lines


def test_install(tmpdir, capfd):
    noop = tmpdir.mkdir('anyci').join('noop.py')
    noop.write("")

    ci_addons.install(str(tmpdir))
    output_lines, _ = captured_lines(capfd)

    for addon in ci_addons.addons():
        assert tmpdir.join(addon).exists()

    assert str(noop) + ' (skipped)' in output_lines
    assert str(tmpdir.join('appveyor', 'patch_vs2008.py')) in output_lines

    ci_addons.install(str(tmpdir), force=True)
    output_lines, _ = captured_lines(capfd)

    assert str(noop) + ' (overwritten)' in output_lines


def test_cli():

    root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    environment = dict(os.environ)
    environment['PYTHONPATH'] = root

    subprocess.check_call(
        "python -m ci_addons",
        shell=True,
        env=environment,
        stderr=subprocess.STDOUT,
        cwd=str(root)
    )




"""
Usage::

    import install_cmake
    install_cmake.install()

"""

import os
import sys

from subprocess import check_call, check_output

DEFAULT_CMAKE_VERSION = "3.5.0"


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[circle:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def install(cmake_version=DEFAULT_CMAKE_VERSION):
    """Download and install CMake into ``/usr/local``."""

    cmake_directory = "/usr/local"

    cmake_exe = os.path.join(cmake_directory, 'bin/cmake')

    if (os.path.exists(cmake_exe)
            and check_output([cmake_exe, '--version']) == cmake_version):
        _log("Skipping download: Found %s (v%s)" % (cmake_exe, cmake_version))
        return

    name = "cmake-{}-Linux-x86_64".format(cmake_version)

    cmake_package = "{}.tar.gz".format(name)

    _log("Downloading ", cmake_package)

    cmake_version_major = cmake_version.split(".")[0]
    cmake_version_minor = cmake_version.split(".")[1]

    check_call([
        "wget", "--no-check-certificate", "--progress=dot",
        "https://cmake.org/files/v{}.{}/{}".format(
            cmake_version_major, cmake_version_minor, cmake_package)
    ])

    _log("Extracting", cmake_package)
    check_call(["tar", "xzf", name + ".tar.gz"])

    _log("Installing", name)
    check_call([
        "sudo", "rsync", "-avz", name + "/", cmake_directory
    ])


if __name__ == '__main__':
    install(sys.argv[1] if len(sys.argv) > 1 else DEFAULT_CMAKE_VERSION)



# -*- coding: utf-8 -*-

"""
scikit-ci-addons is a command line tool and a set of scripts useful to install
prerequisites for building Python extension on CI services.
"""

import os
import shutil
import sys

from subprocess import check_call

__author__ = 'The scikit-build team'
__email__ = 'scikit-build@googlegroups.com'
__version__ = '0.2.0'


def addons():
    """Return all available addons."""

    addons = []

    for dirname, dirnames, filenames in os.walk(home()):

        for v in list(dirnames):
            dirnames.remove(v)
        dirnames += ['anyci', 'appveyor', 'circle', 'travis']

        if dirname == home():
            continue

        for filename in filenames:
            if filename in ['__init__.py']:
                continue
            addon_path = os.path.join(dirname, filename)
            addons.append(os.path.relpath(addon_path, home()))

    return addons


def home():
    """Return directory where all addons can be found."""
    return os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))


def path(addon_name):
    """Return path of ``addon_name``.

    This is particularly useful when the selected addon is not a python
    script and is expected to be used an input to an other tool."""

    addon_path = tmp_addon_path = os.path.join(home(), addon_name)
    if not os.path.exists(tmp_addon_path):
        tmp_addon_path += '.py'
    if not os.path.exists(tmp_addon_path):
        raise RuntimeError("Could not find addon: %s" % addon_path)
    return tmp_addon_path


def install(dst_path, force=False):
    """Copy addons into ``dst_path``.

    By default, existing addons are *NOT* overwritten. Specifying ``force``
    allow to overwrite them.
    """
    dst_path = os.path.normpath(os.path.abspath(dst_path))
    if dst_path == os.path.normpath(home()):
        print("skipping install: target directory already contains addons")
        exit()
    for addon in addons():
        dst_addon_path = os.path.join(dst_path, addon)
        dst_addon_dir = os.path.split(dst_addon_path)[0]
        if not os.path.exists(dst_addon_dir):
            os.makedirs(dst_addon_dir)
        src_addon_path = os.path.join(home(), addon)
        extra = ""
        do_copy = True
        if os.path.exists(dst_addon_path):
            extra = " (skipped)"
            do_copy = False
            if force:
                extra = " (overwritten)"
                do_copy = True
        if do_copy:
            shutil.copy(src_addon_path, dst_addon_path)
        print(dst_addon_path + extra)


def execute(addon_name, arguments=[]):
    """Execute ``addon_name`` with ``arguments``.

    Executable addons are python script.
    """
    cmd = [sys.executable, path(addon_name)] + arguments
    check_call(cmd)



# -*- coding: utf-8 -*-

"""
Ci Addons command line tool
"""

import argparse
import ci_addons
import os


def main():
    """The main entry point to ``ci_addons``.

    This is installed as the script entry point.
    """

    version_str = ("This is scikit-ci-addons version %s, imported from %s\n" %
                   (ci_addons.__version__, os.path.abspath(ci_addons.__file__)))

    parser = argparse.ArgumentParser(description=ci_addons.__doc__)
    parser.add_argument(
        'addon', metavar='ADDON', type=str, nargs='?',
        help='name of addon to execute'
    )
    parser.add_argument(
        'arguments', metavar='ARG', type=str, nargs='*',
        help='addon arguments'
    )
    parser.add_argument(
        "--home", action="store_true",
        help="display directory where all addons can be found"
    )
    parser.add_argument(
        "--list", action="store_true",
        help="list all available addons"
    )
    parser.add_argument(
        "--path", type=str,
        help="display addon path"
    )
    parser.add_argument(
        "--install", type=str,
        help="install addons in the selected directory"
    )
    parser.add_argument(
        "--version", action="version",
        version=version_str,
        help="display scikit-ci-addons version and import information"
    )
    args = parser.parse_args()

    if args.home:
        print(ci_addons.home())
        exit()

    if args.list:
        previous_collection = ""
        for addon in ci_addons.addons():
            current_collection = addon.split(os.path.sep)[0]
            if previous_collection != current_collection:
                print("")
            print(addon)
            previous_collection = current_collection
        exit()

    if args.path is not None:
        print(ci_addons.path(args.path))
        exit()

    if args.install is not None:
        ci_addons.install(args.install)
        exit()

    if all([not getattr(args, arg)
            for arg in ['addon', 'home', 'install', 'list', 'path']]):
        parser.print_usage()
        exit()

    ci_addons.execute(args.addon, args.arguments)


if __name__ == '__main__':
    main()
